# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-04-08 12:20
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_customers', '0005_productpagevisit'),
    ]

    operations = [
        migrations.CreateModel(
            name='WishList',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(max_length=150, verbose_name='Title')),
                ('public', models.BooleanField(default=False, verbose_name='Public')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Changed')),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='pcart_customers.Customer', verbose_name='Customer')),
            ],
            options={
                'verbose_name_plural': 'Wish lists',
                'verbose_name': 'Wish list',
                'ordering': ['title'],
            },
        ),
        migrations.AlterModelOptions(
            name='productpagevisit',
            options={'ordering': ['-changed'], 'verbose_name': 'Product page visit', 'verbose_name_plural': 'Product pages visits'},
        ),
        migrations.AlterUniqueTogether(
            name='wishlist',
            unique_together=set([('title', 'customer')]),
        ),
    ]



from django.shortcuts import render, redirect
from django.conf import settings
from django.http import HttpResponseNotFound, HttpResponseForbidden, Http404
from django.utils.module_loading import import_string
from django.contrib.auth import login
from django.contrib.auth.decorators import login_required
from django.urls import reverse, reverse_lazy
from django.contrib.sites.shortcuts import get_current_site
from django.views.decorators.csrf import csrf_exempt
from django.utils import timezone
from .models import (
    Customer,
    PasswordResetLink,
)
from .utils import get_customer


RESET_LINK_EXPIRATION = 60


@login_required(login_url=reverse_lazy('pcart_customers:login'))
def personal_information(request, template_name='customers/personal_information.html'):
    user = request.user
    customer = get_customer(request)
    context = {
        'user': user,
        'customer': customer,
        'customer_menu': settings.PCART_CUSTOMER_PROFILE_SECTIONS,
    }
    return render(request, template_name, context)


@login_required(login_url=reverse_lazy('pcart_customers:login'))
def personal_information_edit(request, template_name='customers/customer_edit.html'):
    from .forms import EditCustomerForm
    user = request.user
    customer = get_customer(request)

    if request.method == 'POST':
        form = EditCustomerForm(instance=customer, data=request.POST)
        if form.is_valid():
            form.save()
            return redirect(reverse('pcart_customers:personal-information'))
    else:
        form = EditCustomerForm(instance=customer)

    context = {
        'user': user,
        'customer': customer,
        'customer_menu': settings.PCART_CUSTOMER_PROFILE_SECTIONS,
        'form': form,
    }
    return render(request, template_name, context)


@csrf_exempt
@login_required(login_url=reverse_lazy('pcart_customers:login'))
def profile_section(request, slug, sub_slug=None):
    for section in settings.PCART_CUSTOMER_PROFILE_SECTIONS:
        if slug == section['slug']:
            view = import_string(section['view'])
            kwargs = section.get('kwargs', dict())
            if sub_slug is not None:
                kwargs['subsection'] = sub_slug
            return view(request, **kwargs)
    return HttpResponseNotFound('Unknown profile section')


def customer_toolbar(request, template_name='customers/includes/_customer_toolbar_content.html'):
    user = request.user
    context = {
        'user': user,
    }
    return render(request, template_name, context)


def reset_request(request, template_name='customers/password_reset.html'):
    from datetime import timedelta
    from .models import User
    from .forms import UserPasswordResetRequestForm

    if request.user.is_authenticated():
        return HttpResponseForbidden('Access denied.')

    if request.method == 'POST':
        form = UserPasswordResetRequestForm(request.POST)
        if form.is_valid():
            email = form.cleaned_data['email']
            _user = User.objects.get(email=email)
            now = timezone.now()
            expiration_date = now + timedelta(minutes=RESET_LINK_EXPIRATION)
            reset_link = PasswordResetLink.objects.create(user=_user, expiration_date=expiration_date)
            reset_link.send_message()
            return redirect(reverse('pcart_customers:password-reset-done'))
    else:
        form = UserPasswordResetRequestForm()

    context = {
        'form': form,
    }
    return render(request, template_name, context)


def reset_request_done(request, template_name='customers/password_reset_done.html'):
    context = {
        'link_expiration': RESET_LINK_EXPIRATION,
    }
    return render(request, template_name, context)


def reset_confirm(request, slug, template_name='customers/password_reset_confirm.html'):
    from .forms import UserPasswordSetupForm
    if request.user.is_authenticated():
        return HttpResponseForbidden('Access denied.')
    try:
        now = timezone.now()
        reset_link = PasswordResetLink.objects.get(slug=slug, active=True, expiration_date__gte=now)

        if request.method == 'POST':
            form = UserPasswordSetupForm(request.POST)
            if form.is_valid():
                new_password = form.cleaned_data['password2']
                _user = reset_link.user
                _user.set_password(new_password)
                _user.save()

                reset_link.active = False
                reset_link.save()

                login(request, _user)
                return redirect(reverse('pcart_customers:personal-information'))
        else:
            form = UserPasswordSetupForm()

        context = {
            'reset_link': reset_link,
            'user': request.user,
            'form': form,
        }
        return render(request, template_name, context)
    except PasswordResetLink.DoesNotExist:
        return HttpResponseNotFound('Invalid token.')


def login_view(request, template_name='customers/login.html'):
    from .forms import UserAuthForm
    from pcart_cart.utils import get_or_create_cart
    from django.core.exceptions import ObjectDoesNotExist

    redirect_to = request.POST.get('next', request.GET.get('next', ''))
    current_site = get_current_site(request)

    move_cart = 'move-cart' in request.GET

    if request.user.is_authenticated:
        if redirect_to == request.path:
            raise ValueError('Redirection loop for authenticated user detected.')
        return redirect(reverse('pcart_customers:personal-information'))
    elif request.method == 'POST':
        form = UserAuthForm(request, data=request.POST)
        if form.is_valid():
            old_customer = get_customer(request, create=False)
            login(request, form.get_user())
            # If old customer already have a cart than link to the user
            if move_cart and old_customer is not None:
                try:
                    old_cart = old_customer.carts.get(site=current_site)
                    new_customer = get_customer(request)
                    new_cart = get_or_create_cart(current_site, new_customer)
                    new_cart.copy_from(old_cart)
                except ObjectDoesNotExist:
                    pass
            if redirect_to == '':
                redirect_to = reverse('pcart_customers:personal-information')
            return redirect(redirect_to)
    else:
        form = UserAuthForm(request)

    context = {
        'form': form,
        'next': redirect_to,
        'site': current_site,
        'site_name': current_site.name,
    }
    return render(request, template_name, context)


def register(request, template_name='customers/register.html'):
    from .forms import UserRegistrationForm
    if request.user.is_authenticated():
        return redirect(reverse('pcart_customers:personal-information'))

    if request.method == 'POST':
        form = UserRegistrationForm(request.POST)
        if form.is_valid():
            user = form.save()
            customer = get_customer(request, create=False)
            if customer:
                customer.user = user
                customer.save()
            else:
                customer = Customer(user=user)
                customer.save()
            login(request, user)
            return redirect(reverse('pcart_customers:personal-information'))
    else:
        form = UserRegistrationForm()
    context = {
        'form': form,
    }
    return render(request, template_name, context)



# from django.conf import settings
from .models import Customer, User


def get_customer(request, create=True):
    if request.user.is_authenticated:
        try:
            customer = Customer.objects.get(user=request.user)
        except Customer.DoesNotExist:
            if create:
                customer = Customer(user=request.user)
                customer.save()
            else:
                customer = None
    else:
        try:
            customer = Customer.objects.get(session_id=request.session.session_key)
        except Customer.DoesNotExist:
            if create:
                if not request.session.exists(request.session.session_key):
                    request.session.create()
                customer = Customer.objects.create(session_id=request.session.session_key)
                request.session['customer_id'] = str(customer.id)
            else:
                customer = None
    return customer


def create_user_for_customer(customer, email, phone):
    from .models import PasswordResetLink
    user = User.objects.create_user(
        email=email,
        phone=phone,
        password=None,
    )
    user.set_unusable_password()
    user.save()
    customer.user = user
    customer.save()

    # Create a link for setup password
    reset_link = PasswordResetLink.objects.create(user=user)
    # Send email to the new user
    reset_link.send_message()
    return user



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-09 19:30
from __future__ import unicode_literals

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('email', models.EmailField(max_length=255, unique=True, verbose_name='Email')),
                ('phone', models.CharField(max_length=100, unique=True, verbose_name='Phone')),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict, verbose_name='Data')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='Staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='Active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date joined')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name_plural': 'Users',
                'verbose_name': 'User',
            },
        ),
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('session_id', models.CharField(blank=True, default='', max_length=100, verbose_name='Session ID')),
                ('full_name', models.CharField(blank=True, default='', max_length=255, verbose_name='Full name')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Changed')),
                ('user', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='customer', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name_plural': 'Customers',
                'verbose_name': 'Customer',
            },
        ),
    ]



from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.utils.translation import ugettext_lazy as _


class CustomersApphook(CMSApp):
    app_name = "pcart_customers"
    name = _("Customer")

    def get_urls(self, page=None, language=None, **kwargs):
        return ["pcart_customers.urls"]

apphook_pool.register(CustomersApphook)


# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-04-12 09:52
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_customers', '0008_comparisonproduct'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='comparisonproduct',
            name='customer',
        ),
        migrations.RemoveField(
            model_name='comparisonproduct',
            name='product',
        ),
        migrations.RemoveField(
            model_name='comparisonproduct',
            name='variant',
        ),
        migrations.RemoveField(
            model_name='productpagevisit',
            name='customer',
        ),
        migrations.RemoveField(
            model_name='productpagevisit',
            name='product',
        ),
        migrations.RemoveField(
            model_name='productpagevisit',
            name='variant',
        ),
        migrations.AlterUniqueTogether(
            name='wishlist',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='wishlist',
            name='customer',
        ),
        migrations.RemoveField(
            model_name='wishlistedproduct',
            name='product',
        ),
        migrations.RemoveField(
            model_name='wishlistedproduct',
            name='variant',
        ),
        migrations.RemoveField(
            model_name='wishlistedproduct',
            name='wish_list',
        ),
        migrations.DeleteModel(
            name='ComparisonProduct',
        ),
        migrations.DeleteModel(
            name='ProductPageVisit',
        ),
        migrations.DeleteModel(
            name='WishList',
        ),
        migrations.DeleteModel(
            name='WishListedProduct',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-23 18:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_customers', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='customer',
            name='full_name',
        ),
        migrations.AddField(
            model_name='customer',
            name='name',
            field=models.CharField(blank=True, default='', max_length=255, verbose_name='Name'),
        ),
    ]



from django.conf.urls import url
from django.contrib.auth import views as auth_views
from .forms import UserPasswordResetForm
from . import views


urlpatterns = [
    url(r'^login/$', views.login_view, name='login'),
    url(
        r'^logout/$',
        auth_views.logout,
        {'template_name': 'customers/logout.html'},
        name='logout'),
    url(r'^register/$', views.register, name='register'),
    url(
        r'^password-change/$',
        auth_views.password_change,
        {
            'template_name': 'customers/password_change_form.html',
            'post_change_redirect': 'pcart_customers:password-change-done',
        },
        name='password-change'),
    url(
        r'^password-change/done/$',
        auth_views.password_change_done,
        {'template_name': 'customers/password_change_done.html'},
        name='password-change-done'),
    url(r'^password-reset/$', views.reset_request, name='password-reset'),
    url(r'^password-reset/done/$', views.reset_request_done, name='password-reset-done'),
    url(r'^password-reset/(?P<slug>.+)/confirm/$', views.reset_confirm, name='password-reset-confirm'),
    url(r'^personal-information/$', views.personal_information, name='personal-information'),
    url(r'^personal-information/edit/$', views.personal_information_edit, name='personal-information-edit'),
    url(r'^customer-toolbar/$', views.customer_toolbar, name='customer-toolbar'),
    url(r'^(?P<slug>[\w\d-]+)/$', views.profile_section, name='customer-profile-section'),
    url(
        r'^(?P<slug>[\w\d-]+)/(?P<sub_slug>[\w\d-]+)/$',
        views.profile_section, name='customer-profile-section-subsection'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-01 18:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_customers', '0003_passwordresetlink'),
    ]

    operations = [
        migrations.AlterField(
            model_name='passwordresetlink',
            name='slug',
            field=models.CharField(blank=True, max_length=120, unique=True, verbose_name='Slug'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-04-09 09:03
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_catalog', '0013_collection_exclude_properties_filters'),
        ('pcart_customers', '0007_wishlistedproduct'),
    ]

    operations = [
        migrations.CreateModel(
            name='ComparisonProduct',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Changed')),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comparison_products', to='pcart_customers.Customer', verbose_name='Customer')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comparison', to='pcart_catalog.Product', verbose_name='Product')),
                ('variant', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='comparison', to='pcart_catalog.ProductVariant', verbose_name='Variant')),
            ],
            options={
                'verbose_name': 'Comparison product',
                'ordering': ['-added'],
                'verbose_name_plural': 'Comparison products',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-01 17:56
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('pcart_customers', '0002_auto_20170223_2055'),
    ]

    operations = [
        migrations.CreateModel(
            name='PasswordResetLink',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('slug', models.CharField(max_length=120, unique=True, verbose_name='Slug')),
                ('active', models.BooleanField(default=True, verbose_name='Active')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('expiration_date', models.DateTimeField(verbose_name='Expiration date')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='password_reset_link', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Password reset link',
                'verbose_name_plural': 'Password reset links',
            },
        ),
    ]



from django.contrib.auth.backends import ModelBackend
from .models import User


class EmailOrPhoneBackend(ModelBackend):
    def authenticate(self, username=None, password=None, **kwargs):
        from django.db.models import Q
        try:
            user = User.objects.get(Q(email=username)|Q(phone=username))
            if user.check_password(password) and self.user_can_authenticate(user):
                return user
        except User.DoesNotExist:
            pass



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from .models import (
    User,
    Customer,
    PasswordResetLink,
)
from .forms import UserChangeForm, UserCreationForm


class UserAdmin(BaseUserAdmin):
    add_form_template = 'customers/admin/auth/user/add_form.html'
    fieldsets = (
        (None, {
            'fields': (
                'email', 'phone', 'data',
                'password',
            )}),
        (_('Permissions'), {
            'fields': (
                'is_active', 'is_staff',
                'is_superuser',
                'groups',
                'user_permissions',
            )}),
        (_('Important dates'), {
            'fields': ('last_login', 'date_joined')}),
    )
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('email', 'phone', 'password1', 'password2')
        }),
    )

    list_display = (
        'email', 'phone', 'is_staff', 'is_superuser', 'is_active',
        'date_joined', 'last_login', 'has_usable_password',
    )
    list_filter = ('is_staff', 'is_superuser', 'is_active', 'groups')
    search_fields = ('email', 'phone')
    ordering = ('email',)
    date_hierarchy = 'date_joined'
    filter_horizontal = ('groups', 'user_permissions',)

    form = UserChangeForm
    add_form = UserCreationForm
    actions = [
        'activate',
        'deactivate',
        'set_unusable_password',
    ]

    def get_urls(self):
        from django.conf.urls import url
        return [
            url(
                r'^(.+)/change/password/$',
                self.admin_site.admin_view(self.user_change_password),
                name='auth_user_password_change',
            ),
        ] + super().get_urls()

    def activate(self, request, queryset):
        queryset.update(is_active=True)
    activate.short_description = \
        _('Activate')

    def deactivate(self, request, queryset):
        queryset.update(is_active=False)
    deactivate.short_description = \
        _('Deactivate')

    def set_unusable_password(self, request, queryset):
        for q in queryset:
            q.set_unusable_password()
            q.save()
    set_unusable_password.short_description = \
        _('Set unusable password')


admin.site.register(User, UserAdmin)


class CustomerAdmin(admin.ModelAdmin):
    list_display = ('id', 'user', 'session_id', 'name', 'added', 'changed')
    raw_id_fields = ('user',)
    search_fields = ['user', 'session_id', 'name']


admin.site.register(Customer, CustomerAdmin)


class PasswordResetLinkAdmin(admin.ModelAdmin):
    list_display = ('user', 'slug', 'active', 'added', 'expiration_date')
    raw_id_fields = ('user',)
    search_fields = ('user', 'slug')


admin.site.register(PasswordResetLink, PasswordResetLinkAdmin)



from django.db import models
from django.contrib.postgres.fields import JSONField, ArrayField
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site
from django.urls import reverse, NoReverseMatch
from django.conf import settings
import uuid

from django.utils import timezone
from django.contrib.auth.models import (
    AbstractBaseUser,
    BaseUserManager,
    PermissionsMixin,
)


class UserManager(BaseUserManager):
    """
    Creates and saves a User with the given email, phone, password and optional extra info.
    """
    def _create_user(self, email, phone, password,
                     is_staff, is_superuser, **extra_fields):
        """
        Creates and saves a User with the given username, email and password.
        """
        now = timezone.now()

        if not email:
            raise ValueError('The given email must be set')
        if not phone:
            raise ValueError('The given phone must be set')

        email = self.normalize_email(email)
        user = self.model(
            email=email,
            phone=phone,
            is_staff=is_staff, is_active=True,
            is_superuser=is_superuser,
            date_joined=now,
            last_login=now,
            **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email, phone, password=None, **extra_fields):
        return self._create_user(email, phone, password, False, False, **extra_fields)

    def create_superuser(
            self, email, phone, password=None, **extra_fields):
        """
        Creates and saves a superuser with the given email,
        phone and password.
        """
        return self._create_user(email, phone, password, True, True, **extra_fields)


class User(AbstractBaseUser, PermissionsMixin):
    """
    A model which implements the authentication model.

    Email, password and phone are required. Other fields are optional.

    Email and phone fields are used for logging in.
    """
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    email = models.EmailField(_('Email'), max_length=255, unique=True)
    phone = models.CharField(_('Phone'), max_length=100, unique=True)

    data = JSONField(_('Data'), default=dict, blank=True)

    is_staff = models.BooleanField(
        _('Staff status'), default=False, blank=True,
        help_text=_(
            'Designates whether the user can log into this admin site.'))
    is_active = models.BooleanField(
        _('Active'), default=True, blank=True,
        help_text=_('Designates whether this user should be treated as '
                    'active. Unselect this instead of deleting accounts.'))

    date_joined = models.DateTimeField(_('Date joined'), default=timezone.now)

    objects = UserManager()

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['phone']

    class Meta:
        verbose_name = _('User')
        verbose_name_plural = _('Users')

    def __str__(self):
        return self.data.get('full_name', self.email)

    def get_full_name(self):
        return self.data.get('full_name', self.email)

    def get_short_name(self):
        return self.data.get('full_name', self.email)

    def get_email_md5_hash(self):
        import hashlib
        m = hashlib.md5(self.email.lower().encode('utf-8')).hexdigest()
        return m

    def has_usable_password(self):
        return super().has_usable_password()
    has_usable_password.boolean = True


class Customer(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    user = models.OneToOneField(
        User,
        verbose_name=_('User'),
        null=True, blank=True, on_delete=models.CASCADE,
        related_name='customer',
    )
    session_id = models.CharField(_('Session ID'), default='', blank=True, max_length=100)

    name = models.CharField(_('Name'), default='', blank=True, max_length=255)

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    changed = models.DateTimeField(_('Changed'), auto_now=True)

    class Meta:
        verbose_name = _('Customer')
        verbose_name_plural = _('Customers')

    def __str__(self):
        return '%s' % (self.user or self.id)


class PasswordResetLink(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    slug = models.CharField(_('Slug'), unique=True, max_length=120, blank=True)
    user = models.OneToOneField(
        User,
        verbose_name=_('User'), on_delete=models.CASCADE, related_name='password_reset_link',
    )
    active = models.BooleanField(_('Active'), default=True)

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    expiration_date = models.DateTimeField(_('Expiration date'))

    class Meta:
        verbose_name = _('Password reset link')
        verbose_name_plural = _('Password reset links')

    def __str__(self):
        return '%s' % self.user

    def save(self, *args, **kwargs):
        from django.utils import timezone
        from datetime import timedelta

        if not self.slug:
            import uuid
            self.slug = uuid.uuid4().hex

        if not self.expiration_date:
            now = timezone.now()
            self.expiration_date = now + timedelta(days=355)

        super(PasswordResetLink, self).save(*args, **kwargs)

    def get_absolute_url(self):
        try:
            return reverse('pcart_customers:password-reset-confirm', args=[self.slug])
        except NoReverseMatch:
            return '#no-page-for-customers-app'

    def send_message(self, extra_context={}):
        """Send an email with a reset link."""
        from pcart_messaging.utils import create_email_message
        from django.contrib.sites.models import Site
        from django.conf import settings
        sender = settings.DEFAULT_FROM_EMAIL
        site = Site.objects.get_current()
        context = {
            'reset_link': self,
            'site': site,
        }
        context.update(extra_context)
        recipients = [self.user.email]

        email_template_name = 'reset_password' if self.user.has_usable_password() else 'setup_password'
        msg = create_email_message(email_template_name, recipients, sender, context)
        msg.send()



from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.forms import UserCreationForm as BaseUserCreationForm
from django.contrib.auth.forms import UserChangeForm as BaseUserChangeForm
from django.contrib.auth.forms import (
    ReadOnlyPasswordHashField,
    PasswordResetForm,
    AuthenticationForm,
)
from .models import User, Customer


class UserChangeForm(BaseUserChangeForm):
    password = ReadOnlyPasswordHashField(
        label=_('Password'),
        help_text=_(
            'Raw passwords are not stored, so there is no way to see '
            'this user\'s password, but you can change the password '
            'using <a href="password/">this form</a>.'))

    class Meta:
        model = User
        fields = '__all__'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def clean_password(self):
        return self.initial['password']


class UserCreationForm(BaseUserCreationForm):
    """
    A form that creates a user, with no privileges,
    from the given email, phone and password.
    """
    error_messages = {
        'duplicate_email': _("A user with that email already exists."),
        'duplicate_phone': _("A user with that phone already exists."),
        'password_mismatch': _("The two password fields didn't match."),
    }
    password1 = forms.CharField(
        label=_("Password"),
        widget=forms.PasswordInput)
    password2 = forms.CharField(
        label=_("Password confirmation"),
        widget=forms.PasswordInput,
        help_text=_("Enter the same password as above, for verification."))

    class Meta:
        model = User
        fields = '__all__'

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def clean_email(self):
        email = self.cleaned_data['email']
        try:
            User.objects.get(email=email)
        except User.DoesNotExist:
            return email
        raise forms.ValidationError(self.error_messages['duplicate_email'])

    def clean_phone(self):
        phone = self.cleaned_data['phone']
        try:
            User.objects.get(phone=phone)
        except User.DoesNotExist:
            return phone
        raise forms.ValidationError(self.error_messages['duplicate_phone'])

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(
                self.error_messages['password_mismatch'])
        return password2

    def save(self, commit=True):
        user = super().save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user


class EditUserForm(forms.ModelForm):
    class Meta:
        model = User
        fields = ('email', 'phone')


class UserRegistrationForm(UserCreationForm):
    accept_tos = forms.BooleanField(
        label=_(
            'I have read and accept the <a href="/terms/" '
            'rel="nofollow" target="_blank">Terms of Service</a>'),
        initial=False, required=False)

    class Meta:
        model = User
        fields = ('email', 'phone')

    def clean_accept_tos(self):
        t = self.cleaned_data['accept_tos']
        if not t:
            raise forms.ValidationError(
                _('You should accept Terms of Service for registration.'))


class UserPasswordResetForm(PasswordResetForm):
    def get_users(self, email):
        active_users = User.objects.filter(email__iexact=email, is_active=True)
        return (u for u in active_users)


class UserPasswordSetupForm(forms.Form):
    password1 = forms.CharField(
        label=_("Password"),
        widget=forms.PasswordInput)
    password2 = forms.CharField(
        label=_("Password confirmation"),
        widget=forms.PasswordInput,
        help_text=_("Enter the same password as above, for verification."))

    error_messages = {
        'password_mismatch': _("The two password fields didn't match."),
    }

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(
                self.error_messages['password_mismatch'])
        return password2


class UserAuthForm(AuthenticationForm):
    error_messages = {
        'invalid_login': _(
            'Please enter a correct email or phone and password. Note that both '
            'fields may be case-sensitive.'
        ),
        'inactive': _('This account is inactive'),
    }

    def __init__(self, request=None, *args, **kwargs):
        super(UserAuthForm, self).__init__(request, *args, **kwargs)
        self.fields['username'].label = _('Email or phone')


class UserPasswordResetRequestForm(forms.Form):
    email = forms.EmailField(
        label=_('Email'),
        help_text=_('Enter the email your account is registered for.')
    )

    def clean_email(self):
        email = self.cleaned_data['email']
        try:
            User.objects.get(email=email)
            return email
        except User.DoesNotExist:
            raise forms.ValidationError(_('Unknown email.'))


class EditCustomerForm(forms.ModelForm):
    class Meta:
        model = Customer
        fields = ('name',)
